#!/usr/bin/perl
#
# version 1.1

my $llfile = "/tmp/ll.txt";

use Math::Trig;

use POSIX;
use Sys::Syslog qw(:standard :macros);

use strict;

my $debug = 0;

#
# syslog stuff
my $ident = "WXATracking";
my $logopt = "nofatal,pid";
$logopt .= ",perror" if ( $debug == 1);
#my $facility = LOG_USER;
#my $facility = LOG_LOCAL4;
my $facility = LOG_DAEMON;

my $log_is_open = 0;

sub do_close_log() 
{
    closelog()
}

sub do_open_log($)
{
    ($ident) = @_;
    openlog($ident, $logopt, $facility);
    $log_is_open = 1;
}

sub do_exit(@)
{
    my @args = @_;
    do_log(@args);
    do_close_log();
    exit;
}

sub do_log(@)
{
    my @args = @_;

    if ( $log_is_open == 0 ) {
	do_open_log($ident);
    }

    if ( @args == 0 ) {
	syslog(LOG_INFO,"do_log: no arguments sent");
	return;
    }
    chomp(@args);
    syslog(LOG_INFO,@args);
}

sub do_fatal(@)
{
    my (@args) = @_;
    do_log(@args);
    exit 1;
}

#AMVER/PR//
#A/VESSEL NAME/CALL SIGN/IMO #/or just the IMO # "A/1234567//
#B/281330Z NOV//Time stamp current position
#C/4200N/17544W//lat/lon
#E/090//course
#F/200//speed 20.0 knots
#Z/EOR//end of report

sub do_amver_body($$$$$$$$$)
{
    my ($user, $vesselname, $vesselcall, $vesselimo, $time, $lat, $lon, $course, $speed) = @_;

    my $date = uc( strftime("%d%H%M %b",gmtime($time)) );

    my $ns = $lat < 0 ? 'S' : 'N';
    my $ew = $lon < 0 ? 'W' : 'E';
   
    $lat = abs($lat);
    $lon = abs($lon);

    my $dlat = int($lat);
    my $mlat = int(($lat-$dlat)*60);
    my $dlon = int($lon);
    my $mlon = int(($lon-$dlon)*60);
    
    $lat = sprintf("%02d%02d%s",$dlat, $mlat, $ns);
    $lon = sprintf("%03d%02d%s", $dlon, $mlon, $ew);

    $course = sprintf("%03d", int($course));
    $speed = sprintf("%03d", int( $speed*10));

    $vesselname = $user if ( $vesselname eq "" );

    my $aline = "A/$vesselname/$vesselcall/$vesselimo/";
    $aline = "A/$vesselimo//" if ( $vesselimo ne "" );

    my $ret = "

AMVER/PR//
$aline
B/$date//
C/$lat/$lon//
E/$course//
F/$speed//
Z/EOR//

";

    return $ret;

}

sub datetime2sec($)
{
    my ($datetime) = @_;
    my ($date, $time) = split(/ /, $datetime);
    my ($year,$month,$day) = split(/-/,$date);
    my ($hour,$min,$sec)  = split(/:/,$time);
    $month--;
    return timegm($sec, $min, $hour, $day, $month, $year);
}

my $M_PI = 3.14159265358979323846;

sub do_get_old_ll()
{
    return (0,0,0) if ( ! -e "$llfile" );
    open FILE, "<$llfile";
    my $fc = do { local $/; <FILE> };
    chomp($fc);
    my ($utime, $lat, $lon) = split(/,/, $fc);
    close(FILE);
    return ($utime, $lat, $lon);
}

sub do_save_ll($$$)
{
    my ($utime, $lat, $lon) = @_;
    open FILE, ">$llfile";
    print FILE "$utime,$lat,$lon";
    close(FILE);
}

sub round($)
{
    my ($val) = @_;
    return int($val + 0.5);
}

################
# subroutines to compute rhumbline
#
# olddatetime - unix time in seconds
# datetime - unix time in seconds
sub do_cogsog($$$$$$)
{
    my ($datetime, $lat, $lon) = @_;
    my ($olddatetime, $oldlat, $oldlon) = do_get_old_ll();

    my $course = 0;
    my $speed = 0;
    my $distance = 0;
    my $dhours =($datetime - $olddatetime)/3600.0;
    my $distance;

    if ( $olddatetime) {
        ($course, $distance) = rhumbline($oldlat, $oldlon, $lat, $lon);
        $speed = abs($distance/$dhours) if ( 0 != $dhours );
    }

    $speed = round($speed * 0.514444); # convert knots to meters per second
    $course = round($course);
    
    do_save_ll($datetime, $lat, $lon);

    return $olddatetime ? ($course, $speed) : (-1,-1);
}

sub y3($)
{
  my ($Ans) = @_;
  $Ans = fmod($Ans, 360.0);
  return $Ans < 0.0 ? 360.0 + $Ans : $Ans;
}

sub torad($)
{
   my ($x) = @_;
   return ($M_PI*$x)/180.0
}

sub todeg($)
{
   my ($x) = @_;
   return (180*$x)/$M_PI;
}

sub xtan($)
{
    my ($x) = @_;
    $x = torad($x);
    return tan($x);
}

sub xcos($)
{
    my ($x) = @_;
    $x = torad($x);
    return cos($x);
}

sub xsin($)
{
    my ($x) = @_;
    $x = torad($x);
    return sin($x); 
}

sub xatan($)
{
    my ($x) = @_;
    my $spans = atan($x);
    return todeg($spans);
}

sub xlog($)
{
   my ($x) = @_;
   return log10($x);
}

sub rhumbline($$$$)
{
    my ($lat1, $lon1, $lat2, $lon2) = @_;
    my $m1 = merid($lat1);
    my $m2 = merid($lat2);
    my $a = $m1 - $m2;
    my $b = 60.0*dateline($lon1-$lon2);
    $a = .0000006 if ( $a == 0.0 );
    my $spans = xatan($b/$a);
    $spans += 180.0 if ( $a > 0.0 );
    my $c = y3($spans);
    if ( abs($a) > 0.03 ) {
        $a = abs(60*(($lat1 - $lat2)/(xcos($c))));
    } else {
        $a = abs($b*xcos($lat2));
    }
    return ($c, $a);

}

sub dateline($)
{
   my ($x) = @_;
   my $spans = $x;
   $spans = $x + 360.0 if ( $x < -180.0 );
   $spans = $x - 360.0 if ( $x > 180.0 );
   return ($spans);
}

sub merid($)
{
    my ($m) = @_;
    my $a = xsin($m);
    my $spans = 7915.704468 * xlog( xtan(45.0 + $m/2.0) );
    $spans = $spans - 23.0133633*$a - 0.051353*pow($a,3.0);
    $spans = $spans-0.000206*pow($a,5.0);
    return $spans
}

#
# test
#
#my $lat1 = 37.0 + 50.8/60.0;
#my $lon1 = -122 - 25.5/60.0;
#my $lat2 = 34.0 + 52.0/60.0;
#my $lon2 = 139.0 + 42.0/60.0;
#
#my ($course, $distance) = rhumbline($lat1, $lon1, $lat2, $lon2);
#
#print "course = $course, distance = $distance\n";
#
#
# end subroutines to compute rumbline
################

1;
